@@ -744,6 +744,11 @@ bonobo_property_bag_set_value (BonoboPropertyBag *pb,
 		return;
 	}
 
+	if (!(prop->flags & BONOBO_PROPERTY_WRITEABLE)) {
+		bonobo_exception_set (opt_ev, ex_Bonobo_Property_ReadOnlyProperty);
+		return;
+	}
+
 	if (!bonobo_arg_type_is_equal (prop->type, value->_type, opt_ev)) {
 		bonobo_exception_set (opt_ev, ex_Bonobo_Property_InvalidValue);
 		return;
