@@ -103,6 +103,7 @@ efs_stdtype_init ()
 	code = 0;
 	j = 0;
 	k = 0;
+
 	strcpy (prefix, "none");
 	while (std_mime_types[i]) {
 		p = strchr (std_mime_types[i],'/');
@@ -133,13 +134,13 @@ efs_type_init (EFS *efs)
 	guint32 code, i;
 	gint32 l, br;
 	gchar buf[1024], *p;
+
+	if (!efs->typefd) return;
+	if (efs_file_seek (efs->typefd, 0, EFS_SEEK_SET, &i)) return;
 	
 	efs->s2i_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	efs->i2s_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	if (efs_file_open (&efs->typefd, efs->root, ".mime-types", 
-			   EFS_WRITE)) return;
-
 	i = FIRST_USER_TYPE;
 	while (!efs_file_read (efs->typefd, &code, 4, &br)) {
 		if (br != 4) return;
@@ -160,34 +161,50 @@ efs_type_init (EFS *efs)
 	}
 }
 
-static guint32
-integer_type_lookup (EFS *efs, gchar *strtype, gboolean nostd)
+static EFSResult
+integer_type_lookup (EFS *efs, gchar *strtype, gboolean nostd, guint32 *type)
 {
-	guint32 type = 0;
-
-	if (!s2i_std_hash) efs_stdtype_init ();
-	type = (guint32) g_hash_table_lookup (s2i_std_hash, strtype);
-	if (!type && nostd) {
-		if (!efs->s2i_hash) efs_type_init (efs);
-		type = (guint32) g_hash_table_lookup (efs->s2i_hash, strtype);
+	if (!strcmp (strtype, EFS_MIME_DIR) || 
+	    !strcmp (strtype, EFS_MIME_DATA)) {
+		*type = 0;
+		return EFS_ERR_OK;
 	}
 
-	return type;
+	if (!s2i_std_hash) efs_stdtype_init ();
+	*type = (guint32) g_hash_table_lookup (s2i_std_hash, strtype);
+
+	if (!(*type)) {
+		if (!nostd) return EFS_ERR_NOTYPE;
+		if (!efs->s2i_hash) efs_type_init (efs);
+		if (!efs->s2i_hash) return EFS_ERR_INT;
+		*type = (guint32) g_hash_table_lookup (efs->s2i_hash, strtype);
+		if (!(*type)) return EFS_ERR_NOTYPE;
+	}
+
+	return EFS_ERR_OK;
 }
 
-static gchar*
-string_type_lookup (EFS *efs, guint32 type)
+static EFSResult
+string_type_lookup (EFSNode *node, guint32 type, gchar **strtype)
 {
-	gchar *strtype = NULL;
+	EFS *efs = node->efs;
 
-	if (!s2i_std_hash) efs_stdtype_init ();
-	strtype = g_hash_table_lookup (i2s_std_hash, (gpointer)type);
-	if (!strtype) {
-		if (!efs->s2i_hash) efs_type_init (efs);
-		strtype = g_hash_table_lookup (efs->i2s_hash, (gpointer)type);
+	if (type == 0) {
+		if (node->mode&EFS_DIR) *strtype = EFS_MIME_DIR; 
+		else *strtype = EFS_MIME_DATA;
+		return EFS_ERR_OK; 
 	}
 
-	return strtype;
+	if (!i2s_std_hash) efs_stdtype_init ();
+	*strtype = g_hash_table_lookup (i2s_std_hash, (gpointer)type);
+	if (!(*strtype)) {
+		if (!efs->i2s_hash) efs_type_init (efs);
+		if (!efs->i2s_hash) return EFS_ERR_INT;
+		*strtype = g_hash_table_lookup (efs->i2s_hash, (gpointer)type);
+		if (!(*strtype)) return EFS_ERR_NOTYPE;
+	}
+
+	return EFS_ERR_OK;
 }
 
 static guint32
@@ -197,16 +214,15 @@ efs_create_type (EFS *efs, gchar *strtype)
 	gint32 l, l_le;
 	gchar *p;
 
+	if (!efs->s2i_hash | !efs->i2s_hash) return 0;
+	if (!efs->typefd) return 0;
+
 	code = FIRST_USER_TYPE + efs->tc++;
 
 	p = g_strdup (strtype);
 	g_hash_table_insert (efs->s2i_hash, p, (gpointer) code);
 	g_hash_table_insert (efs->i2s_hash, (gpointer) code, p);
-
-	if (!efs->typefd &&
-	    efs_file_open(&efs->typefd, efs->root, ".mime-types", EFS_CREATE))
-		return 0;
-
+	
 	if (efs_file_seek (efs->typefd, 0, EFS_SEEK_END, &cpos)) return 0;
 	code_le = GUINT32_TO_LE (code);
 	if (efs_file_write (efs->typefd, &code_le, 4)) return 0;
@@ -226,13 +242,10 @@ efs_strtype_set (EFSNode *node, gchar *strtype)
 	g_return_val_if_fail (node != NULL, EFS_ERR_INVAL);
 	g_return_val_if_fail (strtype != NULL, EFS_ERR_INVAL);
 
-	if ((type = integer_type_lookup (node->efs, strtype, FALSE)))
+	if (!integer_type_lookup (node->efs, strtype, FALSE, &type))
 		return efs_type_set (node, type);
 
-	if (node->mode&EFS_ROOT)
-		g_warning ("setting non standard type on root efs");
-       
-	if ((type = integer_type_lookup (node->efs, strtype, TRUE)))
+	if (!integer_type_lookup (node->efs, strtype, TRUE, &type))
 		return efs_type_set (node, type);
 
 	if (!(type = efs_create_type (node->efs, strtype))) return EFS_ERR_INT;
@@ -250,43 +263,36 @@ efs_strtype_get (EFSNode *node, gchar **strtype)
 	g_return_val_if_fail (strtype != NULL, EFS_ERR_INVAL);
 
 	if ((res = efs_type_get (node, &type))) return res;
-	
-	if (type == 0) { 
+	if ((res = string_type_lookup(node, type, strtype))) return res;
 
-		if (node->mode&EFS_DIR)
-			*strtype = "x-directory/normal"; 
-		else 
-			*strtype = "application/octet-stream";
-
-		return EFS_ERR_OK; 
-	}
-
-	if ((*strtype=string_type_lookup(node->efs, type))) return EFS_ERR_OK;
-
-	return EFS_ERR_INT;
+	return EFS_ERR_OK;
 }
 
 EFSResult 
 efs_type_lookup (EFSNode *node, gchar *strtype, guint32 *type)
 {
+	EFSResult res;
+
 	g_return_val_if_fail (node != NULL, EFS_ERR_INVAL);
 	g_return_val_if_fail (strtype != NULL, EFS_ERR_INVAL);
 	g_return_val_if_fail (type != NULL, EFS_ERR_INVAL);
 
-	if ((*type = integer_type_lookup (node->efs, strtype, TRUE))) 
-		return EFS_ERR_OK;
+	if ((res = integer_type_lookup (node->efs, strtype, TRUE, type))) 
+		return res;
 
-	return EFS_ERR_INT;
+	return EFS_ERR_OK;
 }
 
 EFSResult
 efs_strtype_lookup (EFSNode *node, guint32 type, gchar **strtype)
 {
+	EFSResult res;
+
 	g_return_val_if_fail (node != NULL, EFS_ERR_INVAL);
 	g_return_val_if_fail (strtype != NULL, EFS_ERR_INVAL);
 
-	if ((*strtype=string_type_lookup(node->efs, type))) return EFS_ERR_OK;
+	if ((res = string_type_lookup(node, type, strtype))) return res;
 
-	return EFS_ERR_INT;
+	return EFS_ERR_OK;
 }
 
