@@ -1,8 +1,6 @@
 #ifndef __BONOBO_CALCULATOR_CONTROL_H__
 #define __BONOBO_CALCULATOR_CONTROL_H__
 
-#include <bonobo/bonobo-control.h>
-
-void             bonobo_calculator_factory_init (void);
+BonoboObject *bonobo_calculator_control_new (void);
 
 #endif /* __BONOBO_CALCULATOR_CONTROL_H__ */
